/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.frameworks.portletcontainer.portalframework;

import java.net.URLDecoder;
import java.util.*;
import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletRequest;
import org.exoplatform.services.portletcontainer.plugins.pc.Constants;

/**
 * Created by The eXo Platform SARL .
 *
 * @author <a href="mailto:lautarul@gmail.com">Roman Pedchenko</a>
 * @version $Id$
 */

public class Helper {

  //decode string
  private static String[] _decode(String s) {
    try {
      return new String[]{URLDecoder.decode(s, "UTF-8")};
    } catch (java.io.UnsupportedEncodingException e) {
      return new String[]{s};
    }
  }

  // puts string array into map
  // if string array has just 1 element, it's placed then as single string
  static public void putMultiply(Map target, String n, String[] param) {
    if (param != null && param.length == 1)
      target.put(n, param[0]);
    else
      target.put(n, param);
  }

  // destiguishes portal service parameters from portlet parameters
  static public void parseParams(HttpServletRequest httpRequest, HashMap portalParams, HashMap portletParams, HashMap propertyParams) {
    System.out.println(" ---------- query string: " + httpRequest.getQueryString());
    Enumeration pNames = httpRequest.getParameterNames();
    while (pNames.hasMoreElements()) {
      String n = (String) pNames.nextElement();
System.out.println(" ---------- parameter: " + n + ": [" + httpRequest.getParameterValues(n)[0] + "]");
      if (n.startsWith(org.exoplatform.Constants.PARAMETER_ENCODER))
        putMultiply(portalParams, n, httpRequest.getParameterValues(n));
      else if (n.startsWith(org.exoplatform.Constants.PROPERTY_ENCODER))
        putMultiply(propertyParams, n, httpRequest.getParameterValues(n));
      else
        putMultiply(portletParams, n, httpRequest.getParameterValues(n));
    }
  }

  // converts action type from string representation to an integer constant
  static public int getActionType(String v) {
    if (v == null)
      return Constants.renderInt;
    if (v.toLowerCase().equals("render"))
      return Constants.renderInt;
    if (v.toLowerCase().equals("action"))
      return Constants.actionInt;
    if (v.toLowerCase().equals("event"))
      return Constants.eventInt;
    if (v.toLowerCase().equals("resource"))
      return Constants.resourceInt;
    return Constants.renderInt;
  }

  // converts portlet mode from string representation
  // to an PortletMode class instance
  static public PortletMode getPortletMode(String v, Collection pms) {
    if (v == null)
      return null;
    if (v == "")
      return null;
    if (v.equalsIgnoreCase(PortletMode.VIEW.toString()))
      return PortletMode.VIEW;
    if (v.equalsIgnoreCase(PortletMode.EDIT.toString()))
      return PortletMode.EDIT;
    if (v.equalsIgnoreCase(PortletMode.HELP.toString()))
      return PortletMode.HELP;
    Iterator i = pms.iterator();
    while (i.hasNext()) {
      PortletMode pm = (PortletMode) i.next();
      if (pm.toString().equalsIgnoreCase(v))
        return pm;
    }
    return PortletMode.VIEW;
  }

  // converts window state from string representation
  // to an WindowState class instance
  static public WindowState getWindowState(String v, Collection wss) {
    if (v == null)
      return null;
    if (v == "")
      return null;
    if (v.equalsIgnoreCase(WindowState.NORMAL.toString()))
      return WindowState.NORMAL;
    if (v.equalsIgnoreCase(WindowState.MAXIMIZED.toString()))
      return WindowState.MAXIMIZED;
    if (v.equalsIgnoreCase(WindowState.MINIMIZED.toString()))
      return WindowState.MINIMIZED;
    Iterator i = wss.iterator();
    while (i.hasNext()) {
      WindowState ws = (WindowState) i.next();
      if (ws.toString().equalsIgnoreCase(v))
        return ws;
    }
    return WindowState.NORMAL;
  }

}
