/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.frameworks.portletcontainer.portalframework;

import java.util.*;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.namespace.QName;
import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.portletcontainer.PortletContainerException;
import org.exoplatform.services.portletcontainer.PortletContainerService;
import org.exoplatform.services.portletcontainer.pci.ActionInput;
import org.exoplatform.services.portletcontainer.pci.ActionOutput;
import org.exoplatform.services.portletcontainer.pci.EventInput;
import org.exoplatform.services.portletcontainer.pci.EventOutput;
import org.exoplatform.services.portletcontainer.pci.Output;
import org.exoplatform.services.portletcontainer.pci.PortletData;
import org.exoplatform.services.portletcontainer.pci.RenderInput;
import org.exoplatform.services.portletcontainer.pci.RenderOutput;
import org.exoplatform.services.portletcontainer.pci.ResourceInput;
import org.exoplatform.services.portletcontainer.pci.ResourceOutput;
import org.exoplatform.services.portletcontainer.pci.model.DisplayName;
import org.exoplatform.services.portletcontainer.pci.model.EventDefinition;
import org.exoplatform.services.portletcontainer.pci.model.Portlet;
import org.exoplatform.services.portletcontainer.pci.model.PortletApp;
import org.exoplatform.services.portletcontainer.plugins.pc.Constants;
import org.exoplatform.services.portletcontainer.plugins.pc.event.EventImpl;

/**
 * Created by The eXo Platform SARL .
 *
 * @author <a href="mailto:lautarul@gmail.com">Roman Pedchenko</a>
 * @version $Id$
 */

public class PortalFramework {

  protected ExoContainer portalContainer = null;

  protected PortletContainerService service = null;

  protected Map allPortletMetaData = null;

  protected HashMap wins = null;
  protected HashMap eventDelivery = null;
  protected HashMap publicParams = null;
  protected HashMap portalParams = null;
  protected HashMap portletParams = null;
  protected HashMap propertyParams = null;
  protected Map actionParams = null;
  protected Map resourceParams = null;
  protected HashSet changes = null;

  protected String target = null;
  protected int action = -1;
  protected String cntType = null;
  protected List events = null;
  protected String redirect = null;

  public PortalFramework(ExoContainer cnt) {
    portalContainer = cnt;
  }

  // returns name of parent portal
  public String getPortalName() {
    return portalContainer.getContext().getName();
  }

  // returns name of target portlet after parsing incoming http request
  public String getTarget() {
    return target;
  }

  // returns name of requested action after parsing incoming http request
  public int getAction() {
    return action;
  }

  // returns redirect path (if any) after processing of the portlet request
  public String getRedirect() {
    return redirect;
  }

  // returns set portlets that were changed after event processing
  public HashSet getChanges() {
    return changes;
  }

  // returns set of all portlets currently registered in the portlet container
  public Set getPortletNames() {
    return allPortletMetaData.keySet();
  }

  // returns collection of supported portlet modes for specified portlet
  public Collection getPortletModes(String pan, String pn) {
    return service.getPortletModes(pan, pn, cntType);
  }

  // returns collection of supported window states for specified portlet
  public Collection getWindowStates(String pan, String pn) {
    return service.getWindowStates(pan, pn, cntType);
  }

  // returns display name of specified portlet
  public String getPortletDisplayName(String plt) {
    PortletData portlet = (PortletData) allPortletMetaData.get(plt);
    if (portlet.getDisplayName() == null)
      return null;
    if (portlet.getDisplayName().size() == 0)
      return null;
    return ((DisplayName) portlet.getDisplayName().get(0)).getDisplayName();
  }

  // returns WindowID object for specified portlet
  public WindowID2 getWindowID(String plt) {
    return (WindowID2) wins.get(plt);
  }

  // returns map of portal parameters after parsing incoming http request
  public HashMap getPortalParams() {
    return portalParams;
  }

  // returns map of portlet parameters after parsing incoming http request
  public HashMap getPortletParams() {
    return portletParams;
  }

  // parses incoming http request parameters, sets up requested action, target portlet (if any)
  // for the requested action, and changes portlet mode and window state for the target portlet
  // if requested
  public void setParams(HttpServletRequest httpRequest, String cntType) {
    this.cntType = cntType;
    portalParams = new HashMap();
    portletParams = new HashMap();
    actionParams = null;
    resourceParams = null;
    propertyParams = null;
    events = new ArrayList();
    redirect = null;
    changes = new HashSet();

    Helper.parseParams(httpRequest, portalParams, portletParams, propertyParams);
    target = (String) portalParams.get(org.exoplatform.Constants.COMPONENT_PARAMETER);
    action = Helper.getActionType((String) portalParams.get(org.exoplatform.Constants.TYPE_PARAMETER));

    if (target == null)
      action = Constants.renderInt;

    if (!portalParams.isEmpty() && portalParams.containsKey(org.exoplatform.Constants.WINDOW_STATE_PARAMETER)) {
      if (((String) portalParams.get(org.exoplatform.Constants.WINDOW_STATE_PARAMETER)).equals(WindowState.MINIMIZED)) {
        Set set = wins.keySet();
        Iterator it = set.iterator();
        while (it.hasNext()) {
          String key = (String) it.next();
          WindowID2 window = (WindowID2) wins.get(key);
          window.setWindowState(WindowState.MINIMIZED);
        }
      }
    }

    String[] pn = StringUtils.split(target, "/");

    if (target != null) {
      WindowID2 win = (WindowID2) wins.get(target);

      // set MODE and STATE
      PortletMode portletMode = Helper.getPortletMode(
          (String) portalParams.get(org.exoplatform.Constants.PORTLET_MODE_PARAMETER),
          getPortletModes(pn[0], pn[1]));
      
      WindowState windowState = Helper.getWindowState(
          (String) portalParams.get(org.exoplatform.Constants.WINDOW_STATE_PARAMETER),
          getWindowStates(pn[0], pn[1]));

      if (portletMode != null) {
        changes.add(target);
        win.setPortletMode(portletMode);
      }

      if (windowState != null) {
        changes.add(target);
        win.setWindowState(windowState);
      }
    }
  }

  public void init(HttpSession httpSession) {
    service =
      (PortletContainerService) portalContainer.getComponentInstanceOfType(PortletContainerService.class);
    initMaps();
  }

  protected void initMaps() {
    allPortletMetaData = service.getAllPortletMetaData();
    buildEventDeliveryAndPublicParamsMaps(allPortletMetaData);
    createOrUpdateWins(allPortletMetaData);
  }

  // [re]creates WindowID map
  // for newly appeared portlets it creates new WindowID instance
  // for already used ones it just copies it
  // for deleted ones it just abandons appropriate instances
  protected void createOrUpdateWins(Map allPortletMetaData) {
    HashMap newWins = new HashMap();
    if (wins == null)
      wins = new HashMap();
    Set keys = allPortletMetaData.keySet();
    Iterator i = keys.iterator();
    while(i.hasNext()) {
      String key = (String) i.next();
      String portletName = key.replace('.' , '/');
      String[] ss = StringUtils.split(portletName, "/");

      if (wins.get(portletName) == null) {
        WindowID2 windowID = new WindowID2();
        windowID.setOwner(org.exoplatform.Constants.ANON_USER);
        windowID.setPortletApplicationName(ss[0]);
        windowID.setPortletName(ss[1]);
        windowID.setUniqueID(portletName);
        windowID.setPersistenceId(ss[0] + "II" + ss[1]);
        windowID.setPortletMode(PortletMode.VIEW);
        windowID.setWindowState(WindowState.NORMAL);
        newWins.put(portletName, windowID);
      } else
        newWins.put(portletName, wins.get(portletName));
    }
    wins = newWins;
  }

  // generates maps for event and public parameters delivery
  protected void buildEventDeliveryAndPublicParamsMaps(Map allPortletMetaData) {
    eventDelivery = new HashMap();
    publicParams = new HashMap();
    Iterator i = allPortletMetaData.keySet().iterator();
    while(i.hasNext()) {
      String pname = (String) i.next();
      PortletData portlet = (PortletData) allPortletMetaData.get(pname);
      // public params
      List srp = portlet.getSupportedPublicRenderParameter();
      if (srp != null) {
        publicParams.put(pname, srp);
      }
      // event delivery
      List spe = portlet.getSupportedProcessingEvent();
      if (spe == null)
        continue;
      Iterator i1 = spe.iterator();
      while (i1.hasNext()) {
        QName ename = (QName) i1.next();
        List dl = (List) eventDelivery.get(ename);
        if (dl == null) {
          dl = new ArrayList();
          eventDelivery.put(ename, dl);
        }
        dl.add(pname);
      }
    }
  }

  // for specified portlet adds public params that are supported by the portlet
  protected HashMap fillPublicParams(String pname, HashMap params) {
    HashMap result = new HashMap();
    List srp = (List) publicParams.get(pname);
    if (srp != null) {
      Iterator i = params.keySet().iterator();
      while (i.hasNext()) {
        String rname = (String) i.next();
        if (srp.contains(rname)) {
          result.put(rname, params.get(rname));
        }
      }
    }
    return result;
  }

  protected boolean getPortletEscapeXml(String target) {
    try {
      return ((PortletData) allPortletMetaData.get(target)).getEscapeXml();
    } catch (Exception e) {
      e.printStackTrace();
      return false;
    }
  }

  // checks if an event that is about to be delivered has its payload type appropriate to declared
  protected boolean checkEventValueType(EventInfo event) {
    if (event.value.getValue() == null)
      return false;
    String[] hs = StringUtils.split(event.target, "/");
    try {
      return service.isEventPayloadTypeMatches(hs[0], event.name.toString(), event.value.getValue());
    } catch (NullPointerException npe) {
      return true;
    }
  }

  // adds events for delivery just after checking of their type
  protected void addEvents(List events, Map newEvents) {
    for (Iterator i = newEvents.keySet().iterator(); i.hasNext();) {
      QName name = (QName) i.next();
      List pl = (List) eventDelivery.get(name);
      if (pl == null) {
        pl = new ArrayList();
        eventDelivery.put(name, pl);
      }
      Object value = (Object) newEvents.get(name);
      for (Iterator i1 = pl.iterator(); i1.hasNext();) {
        String pname = (String) i1.next();
        EventInfo event = new EventInfo();
        event.name = name;
        event.value = new EventImpl(name, value);
        event.target = pname;
        if (checkEventValueType(event))
          events.add(event);
      }
    }
  }

  // --- input objects creators ---

  // creates ResourceInput instance and fills it with appropriate values
  public ResourceInput createResourceInput(ServletContext ctx) {
    ResourceInput resourceInput = new ResourceInput();
    WindowID2 win = (WindowID2) wins.get(target);
    ArrayList lo = new ArrayList();
    lo.add(new Locale("en"));
    resourceInput.setLocales(lo);
    resourceInput.setEscapeXml(getPortletEscapeXml(target));
    resourceInput.setWindowID(win);
    resourceInput.setBaseURL("/" + ctx.getServletContextName() + "/?"
      + org.exoplatform.Constants.COMPONENT_PARAMETER + "=" + win.getUniqueID());
    resourceInput.setUserAttributes(new HashMap());
    resourceInput.setMarkup(cntType);
    resourceInput.setRenderParameters(portletParams);
    resourceInput.setPortletMode(win.getPortletMode());
    resourceInput.setWindowState(win.getWindowState());
    resourceInput.setResourceID((String) portalParams.get(org.exoplatform.Constants.RESOURCE_ID_PARAMETER));
    return resourceInput;
  }

  // creates ActionInput instance and fills it with appropriate values
  public ActionInput createActionInput(ServletContext ctx) {
    ActionInput actionInput = new ActionInput();
    WindowID2 win = (WindowID2) wins.get(target);
    ArrayList lo = new ArrayList();
    lo.add(new Locale("en"));
    actionInput.setLocales(lo);
    actionInput.setEscapeXml(getPortletEscapeXml(target));
    actionInput.setWindowID(win);
    actionInput.setBaseURL("/" + ctx.getServletContextName() + "/?"
      + org.exoplatform.Constants.COMPONENT_PARAMETER + "=" + win.getUniqueID());
    actionInput.setUserAttributes(new HashMap());
    actionInput.setMarkup(cntType);
    actionInput.setPortletMode(win.getPortletMode());
    actionInput.setWindowState(win.getWindowState());
    actionInput.setStateChangeAuthorized(true);
    return actionInput;
  }

  // creates EventInput instance and fills it with appropriate values
  public EventInput createEventInput(ServletContext ctx, EventInfo event) {
    changes.add(event.target);
    EventInput eventInput = new EventInput();
    WindowID2 win = (WindowID2) wins.get(event.target);
    ArrayList lo = new ArrayList();
    lo.add(new Locale("en"));
    eventInput.setLocales(lo);
    eventInput.setEscapeXml(getPortletEscapeXml(event.target));
    eventInput.setWindowID(win);
    eventInput.setBaseURL("/" + ctx.getServletContextName() + "/?"
      + org.exoplatform.Constants.COMPONENT_PARAMETER + "=" + win.getUniqueID());
    eventInput.setUserAttributes(new HashMap());
    eventInput.setMarkup(cntType);
    eventInput.setPortletMode(win.getPortletMode());
    eventInput.setWindowState(win.getWindowState());
    eventInput.setEvent(event.value);
    return eventInput;
  }

  // creates RenderInput instance and fills it with appropriate values
  public RenderInput createRenderInput(ServletContext ctx, String plt) {
    RenderInput renderInput = new RenderInput();
    WindowID2 win = (WindowID2) wins.get(plt);
    ArrayList lo = new ArrayList();
    lo.add(new Locale("en"));
    renderInput.setLocales(lo);
    renderInput.setEscapeXml(getPortletEscapeXml(plt));
    renderInput.setWindowID(win);
    renderInput.setBaseURL("/" + ctx.getServletContextName() + "/?"
      + org.exoplatform.Constants.COMPONENT_PARAMETER + "=" + win.getUniqueID());
    renderInput.setUserAttributes(new HashMap());
    renderInput.setMarkup(cntType);
    List pubs = (List) publicParams.get(plt);
    HashSet pubNames = (pubs == null) ? new HashSet() : new HashSet(pubs);
    renderInput.setPublicParamNames(pubNames);
    if (target != null && target.equals(plt))
      renderInput.setRenderParameters(portletParams);
    else
      renderInput.setRenderParameters(fillPublicParams(plt, portletParams));
    if (actionParams != null)
      renderInput.getRenderParameters().putAll(actionParams);
    if (resourceParams != null)
      renderInput.getRenderParameters().putAll(resourceParams);
    renderInput.setPortletMode(win.getPortletMode());
    renderInput.setWindowState(win.getWindowState());
    return renderInput;
  }

  // --- portlet method callers ---

  // calls serveResource() for target portlet and parses its output
  public ResourceOutput serveResource(HttpServletRequest httpRequest, HttpServletResponse httpResponse,
      ResourceInput resourceInput) throws PortletContainerException {
    
    ResourceOutput o = service.serveResource(httpRequest, httpResponse, resourceInput);
    resourceParams = o.getRenderParameters();
    addEvents(events, o.getEvents());
    redirect = (String) o.getProperties().get(Output.SEND_REDIRECT);
    WindowID2 win = (WindowID2) wins.get(target);
    if (o.getNextMode() != null)
      win.setPortletMode(o.getNextMode());
    if (o.getNextState() != null)
      win.setWindowState(o.getNextState());
    return o;
  }

  // calls processAction() for target portlet and parses its output
  public ActionOutput processAction(HttpServletRequest httpRequest, HttpServletResponse httpResponse,
    ActionInput actionInput) throws PortletContainerException {
    ActionOutput o = service.processAction(httpRequest, httpResponse, actionInput);
    actionParams = o.getRenderParameters();
    addEvents(events, o.getEvents());
    redirect = (String) o.getProperties().get(Output.SEND_REDIRECT);
    WindowID2 win = (WindowID2) wins.get(target);
    if (o.getNextMode() != null)
      win.setPortletMode(o.getNextMode());
    if (o.getNextState() != null)
      win.setWindowState(o.getNextState());
    return o;
  }

  // calls processEvent() for target portlet and parses its output
  public EventOutput processEvent(HttpServletRequest httpRequest, HttpServletResponse httpResponse,
    EventInput eventInput, EventInfo event) throws PortletContainerException {
    EventOutput o = service.processEvent(httpRequest, httpResponse, eventInput);
    addEvents(events, o.getEvents());
    WindowID2 win = (WindowID2) wins.get(event.target);
    if (o.getNextMode() != null)
      win.setPortletMode(o.getNextMode());
    if (o.getNextState() != null)
      win.setWindowState(o.getNextState());
    return o;
  }

  // calls render() for target portlet and parses its output
  public RenderOutput render(HttpServletRequest httpRequest, HttpServletResponse httpResponse,
    RenderInput renderInput) throws PortletContainerException {
    RenderOutput o = service.render(httpRequest, httpResponse, renderInput);
    return o;
  }

  // --- ---

  // subsequently delivers generated events
  public void dispatchEvents(ServletContext ctx, HttpServletRequest httpRequest, HttpServletResponse httpResponse)
    throws PortletContainerException {
    while (events.size() > 0) {
      EventInfo event = (EventInfo) events.get(0);
      events.remove(0);
      EventInput eventInput = createEventInput(ctx, event);
      EventOutput o = processEvent(httpRequest, httpResponse, eventInput, event);
    }
  }

}
