/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portletcontainer.pci.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.portlet.filter.FilterChain;
import javax.portlet.PortletSession;
import javax.xml.namespace.QName;


import org.exoplatform.services.portletcontainer.PortletContainerConstants;
import org.exoplatform.Constants;
import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;


/**
 * Jul 11, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: Portlet.java,v 1.1 2004/07/13 02:31:13 tuan08 Exp $
 */
public class Portlet {
	private List								description;
	private String							portletName;
	private List								displayName = new ArrayList() ;
	private String							portletClass;
	private List								initParam;
	private Integer        			caching;
	private List								supports;
	private List								supportedLocale;
	private String							resourceBundle;
	private PortletInfo					portletInfo;
	private ExoPortletPreferences	portletPreferences;
	private List								securityRoleRef;
	// portlet api 2.0
	private String							id;
  private List supportedProcessingEvent;
  private List supportedPublishingEvent;
  private List supportedPublicRenderParameter;
  private Map<String,String[]> containerRuntimeOption;
  private List sharedSessionAttribute;
  private FilterChain filterChain = null;
  private PortletApp application;
	//exo extension
	private String							globalCache;
	private Boolean             escapeXml;// = true;
  private int              portletSessionScope;
  private Map <String,String[]> localeEncodingList;

  public PortletApp getApplication() {
    return application;
  }
  
  public void setApplication(PortletApp app) {
    application = app;
  }

  public void setEscapeXml(Boolean escapeXml) {
    this.escapeXml = escapeXml;
  }
  
  public void setEscapeXml(boolean escapeXml) {
    this.escapeXml = new Boolean(escapeXml);
  }
  
  public Boolean getEscapeXml() {
    if (escapeXml != null) 
      return escapeXml;
    if (getContainerRuntimeOption() != null) {
      String[] valuesPortlet = (String[])getContainerRuntimeOption().get("javax.portlet.escapeXml");
      if (valuesPortlet != null)
        return new Boolean(valuesPortlet[0]);
    }
    String[] valuesApplication = (String[])application.getContainerRuntimeOption().get("javax.portlet.escapeXml");
    if (valuesApplication != null)
      return new Boolean(valuesApplication[0]);
    return Boolean.TRUE;
  }
  
  
  public int getPortletSessionScope() {
    
    if (getContainerRuntimeOption() != null) {
      String[] valuesPortlet = (String[])getContainerRuntimeOption().get("javax.portlet.includedPortletSessionScope");
      if (valuesPortlet != null && valuesPortlet[0].equals(PortletContainerConstants.PORTLET_SCOPE))
        return PortletSession.PORTLET_SCOPE;
    }
    String[] valuesApplication = (String[])application.getContainerRuntimeOption().get("javax.portlet.includedPortletSessionScope");
    if (valuesApplication != null && valuesApplication[0].equals(PortletContainerConstants.PORTLET_SCOPE))
        return PortletSession.PORTLET_SCOPE;
    
    return PortletSession.APPLICATION_SCOPE;
  }
  
  
  public FilterChain getFilterChain() {
    return filterChain;
  }

  public void setFilterChain(FilterChain filterChain) {
    this.filterChain = filterChain;
  }

	public List getDescription() {
    if(description == null) return Constants.EMPTY_LIST ;  
		return description;
	}

  public String getDescription(String lang) {
    return Util.getDescription(lang, description) ;
  }
  
	public void setDescription(List description) {
		this.description = description;
	}
	
  public void addDescription(Description desc) {
    if(description == null) description = new ArrayList() ;
  	this.description.add(desc) ;
  }
  
	public List getDisplayName() {
		return displayName;
	}
  
  public void addDisplayName(DisplayName name) {
  	this.displayName.add(name) ;
  }

	public void setDisplayName(List displayName) {
		this.displayName = displayName;
	}

	public Integer getCaching() {
		return caching;
	}

	public void setCaching(Integer caching) {
		this.caching = caching;
	}

	public String getGlobalCache() {
		return globalCache;
	}

	public void setGlobalCache(String globalCache) {
		this.globalCache = globalCache;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public List getInitParam() {
    if(initParam == null) return Constants.EMPTY_LIST ;  
		return initParam;
	}

	public void setInitParam(List initParam) {
		this.initParam = initParam;
	}
  
  public void addInitParam(InitParam param) {
    if(initParam == null) initParam = new ArrayList() ;
    
    if (initParam.contains(param)) {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("Duplicate field \"init-param\" in portlet description");
   }
   else {
     this.initParam.add(param); 
    }
    
  	
  }

	public String getPortletClass() {
		return portletClass;
	}

	public void setPortletClass(String portletClass) {
		this.portletClass = portletClass;
	}

	public PortletInfo getPortletInfo() {
		return portletInfo;
	}

	public void setPortletInfo(PortletInfo portletInfo) {
		this.portletInfo = portletInfo;
	}

	public String getPortletName() {
		return portletName;
	}

	public void setPortletName(String portletName) {
		this.portletName = portletName;
	}

	public ExoPortletPreferences getPortletPreferences() {
		return portletPreferences;
	}

	public void setPortletPreferences(ExoPortletPreferences portletPreferences) {
		this.portletPreferences = portletPreferences;
	}

	public String getResourceBundle() {
		return resourceBundle;
	}

	public void setResourceBundle(String resourceBundle) {
		this.resourceBundle = resourceBundle;
	}

	public List getSecurityRoleRef() {
    if(securityRoleRef == null) return Constants.EMPTY_LIST ;  
		return securityRoleRef;
	}

	public void setSecurityRoleRef(List securityRoleRef) {
		this.securityRoleRef = securityRoleRef;
	}
  
  public void addSecurityRoleRef(SecurityRoleRef ref) {
   if(securityRoleRef == null ) securityRoleRef = new ArrayList() ;
   
   if (securityRoleRef.contains(ref)) {
     Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
     log.error("Duplicate field \"security-role-ref\" in portlet description");
  }
  else {
    this.securityRoleRef.add(ref) ; 
   }
   
    
  }

	public List getSupportedLocale() {
    if(supportedLocale == null) return Constants.EMPTY_LIST ;  
		return supportedLocale;
	}

	public void setSupportedLocale(List supportedLocale) {
		this.supportedLocale = supportedLocale;
	}
	
  public void addSupportedLocale(String value) {
   if(supportedLocale == null) supportedLocale = new ArrayList() ;
   this.supportedLocale.add(value) ; 
  }
  
	public List getSupports() {
    if(supports == null) return Constants.EMPTY_LIST ;
    
    
    
		return supports;
	}

	public void setSupports(Supports s) {
    if(supports == null) this.supports = new ArrayList() ;
    
    if (supports.contains(s)) {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("Duplicate field \"supports\" in portlet description");
   }
   else {
     this.supports.add(s) ; 
    }
    
    
	}
  
  public void addSupports(Supports s) {
    if(supports == null) this.supports = new ArrayList() ;
    this.supports.add(s) ;
  }

  public List getSharedSessionAttribute() { return sharedSessionAttribute; }
  
  public void addSharedSessionAttribute(SharedSessionAttribute ssa) {
    if (sharedSessionAttribute == null)
      sharedSessionAttribute = new ArrayList();
  	this.sharedSessionAttribute.add(ssa);
  }

  public List getSupportedProcessingEvent() { return supportedProcessingEvent; }
  
  public void addSupportedProcessingEvent(QName spe) {
    if (supportedProcessingEvent == null)
      supportedProcessingEvent = new ArrayList();
    if (supportedProcessingEvent.contains(spe)) {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("Duplicate field \"supported-processing-event\" in portlet description");
   }
   else {
     this.supportedProcessingEvent.add(spe); 
    }
  }


  public List getSupportedPublishingEvent() { return supportedPublishingEvent; }
  
  public void addSupportedPublishingEvent(QName spe) {
    if (supportedPublishingEvent == null)
      supportedPublishingEvent = new ArrayList();
    
    if (supportedPublishingEvent.contains(spe)) {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("Duplicate field \"supported-publishing-event\" in portlet description");
   }
   else {
    this.supportedPublishingEvent.add(spe); 
    }
    
  
  }

  public List getSupportedPublicRenderParameter() { return supportedPublicRenderParameter; }
  
  public void addSupportedPublicRenderParameter(String srp) {
    if (supportedPublicRenderParameter == null)
      supportedPublicRenderParameter = new ArrayList();
    
    if (supportedPublicRenderParameter.contains(srp)) {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("Duplicate field \"supported-public-render-parameter\" in portlet description");
   }
   else {
     this.supportedPublicRenderParameter.add(srp); 
    }
    
  	
  }
  
  public void setContainerRuntimeOption(Map containerRuntimeOption) {
    if (containerRuntimeOption == null)
      containerRuntimeOption = new HashMap<String,String[]>();
    this.containerRuntimeOption = containerRuntimeOption;
  }

  public Map getContainerRuntimeOption() { 
    return containerRuntimeOption; 
  }

  public void addContainerRuntimeOption(String name, String[] value) {
    if (containerRuntimeOption == null)
      containerRuntimeOption = new HashMap<String,String[]>();
    this.containerRuntimeOption.put(name,value);
  }
  
  public void addContainerRuntimeOption(Map containerRuntimeOption) {
    if (containerRuntimeOption == null)
      containerRuntimeOption = new HashMap<String,String[]>();
    this.containerRuntimeOption.putAll(containerRuntimeOption);
  }
    
}