/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portletcontainer.pci.model;

import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import java.util.Map;

import org.exoplatform.Constants;
import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;

/**
 * Jul 11, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: PortletApp.java,v 1.1 2004/07/13 02:31:13 tuan08 Exp $
 */
public class PortletApp {
	private List portlet;
  private String version;
  private List customWindowState;
  private List customPortletMode;
  private List securityConstraint;
  private List userAttribute;
  private List eventDefinition;
  private List sharedSessionAttribute;
  private List publicRenderParameter;
	private List filter;
	private List filterMapping;
  private String id;
  //Portlet spec 2 add:
  private String resourceBundle;  
  private Map<String,String[]> containerRuntimeOption;
  private boolean ver2;
  private String defaultNamespace = javax.xml.XMLConstants.NULL_NS_URI;
  
  public PortletApp() {
  	portlet = new ArrayList();
    customWindowState = new ArrayList();
    customPortletMode = new ArrayList();
    securityConstraint = new ArrayList();
    userAttribute = new ArrayList();
    eventDefinition = new ArrayList();
    sharedSessionAttribute = new ArrayList();
    publicRenderParameter = new ArrayList();
    containerRuntimeOption = new HashMap<String,String[]>();
  }
  
  public void setDefaultNamespace(String namespace) {
    defaultNamespace = namespace;
  }
  
  public String getDefaultNamespace() {
    return defaultNamespace;
  }
  
  public boolean getVer2() { return this.ver2; }
  public void setVer2(boolean value) { this.ver2 = value; }

  public List getPortlet() { return this.portlet; }
  
  public void addPortlet(Portlet p) {
    
    if (portlet.contains(p)) {
       Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
       log.error("Duplicate field \"Portlet-Name\" in portlet-application description");
    }
    else {
        this.portlet.add(p); 
        p.setApplication(this);
        }
    }
  
  public String getVersion() { return this.version; }
  public void setVersion(String value) { this.version = value; }

	public List getFilter() {
    if(filter == null) return Constants.EMPTY_LIST ;  
		return filter;
	}

	public void setFilter(List filter) {
		this.filter = filter;
	}

  public void addFilter(Filter f) {
    if (f != null) {
      if(filter == null) filter = new ArrayList();  
      
      if (filter.contains(f)) {
        Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
        log.error("Duplicate field \"filte\" in portlet-application description");
     }
     else {
       this.filter.add(f) ;
      }
      
     
    }
  }
  
	public List getFilterMapping() {
    if(filterMapping == null) return Constants.EMPTY_LIST ;  
		return filterMapping;
	}

	public void setFilterMapping(List filter) {
		this.filterMapping = filterMapping;
	}

  public void addFilterMapping(FilterMapping f) {
   if(filterMapping == null) filterMapping = new ArrayList();  
   this.filterMapping.add(f) ; 
  }
  
  public List getCustomWindowState() { return this.customWindowState; }
  
  public void addCustomWindowState(CustomWindowState value) {
    
    if (customWindowState.contains(value)) {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("Duplicate field \"custom-window-state\" in portlet-application description");
   }
   else {
     this.customWindowState.add(value); 
    }
    
    
  }

  public List getCustomPortletMode() { return this.customPortletMode;}
  
  public void addCustomPortletMode(CustomPortletMode mode) {
    if (customPortletMode.contains(mode)) {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("Duplicate field \"custom-window-state\" in portlet-application description");
   }
   else {
     this.customPortletMode.add(mode); 
    }
   }

  public List getSecurityConstraint() { return this.securityConstraint; }
  public void addSecurityConstraint(SecurityConstraint sc) {
  	this.securityConstraint.add(sc);
  }
  
  public List getUserAttribute() { return userAttribute; }
  
  public void addUserAttribute(UserAttribute att) {
    
    if (userAttribute.contains(att)) {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("Duplicate field \"user-attribute\" in portlet-application description");
   }
   else {
     this.userAttribute.add(att); 
    }
    
  	
  }

  public List getEventDefinition() { return eventDefinition; }
  
  public void addEventDefinition(EventDefinition edef) {
    
    if (eventDefinition.contains(edef)) {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("Duplicate field \"event-definition\" in portlet-application description");
   }
   else {
     this.eventDefinition.add(edef); 
    }
  	
  }

  public List getSharedSessionAttribute() { return sharedSessionAttribute; }
  public void addSharedSessionAttribute(SharedSessionAttribute ssa) {
  	this.sharedSessionAttribute.add(ssa);
  }

  public List getPublicRenderParameter() { return publicRenderParameter; }
  public void addPublicRenderParameter(PublicRenderParameter srp) {
    
    if (publicRenderParameter.contains(srp)) {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("Duplicate field \"public-render-parameter\" in portlet-application description");
   }
   else {
     this.publicRenderParameter.add(srp); 
    }
    
  	
  }
  
  public void setContainerRuntimeOption(Map containerRuntimeOption) {
    this.containerRuntimeOption = containerRuntimeOption;
  }

  public Map getContainerRuntimeOption() { 
    return containerRuntimeOption; 
  }

  public void addContainerRuntimeOption(String name, String[] value) {
    this.containerRuntimeOption.put(name,value);
  }
  
  public void addContainerRuntimeOption(Map containerRuntimeOption) {
    this.containerRuntimeOption.putAll(containerRuntimeOption);
  }
  
  public String getId() { return this.id; }

  public void setId(String value) { this.id = value; }
  
  public String getResourceBundle() {
    return resourceBundle;
  }

  public void setResourceBundle(String resourceBundle) {
    this.resourceBundle = resourceBundle;
  }
}